local l = BixLib
local C = Growler_Constants
local S = Growler_Stable
local SL = Growler_Slash
local _,L = ...

Growler_Options = CreateFrame("FRAME", "GrowlerOptions", UIParent)
local O = Growler_Options

O.CreateOption = function(name,text,dbKey,frametype,o,insetlevel,handler,...)
	local args = {...}
	local template = "ChatConfigCheckButtonTemplate"
	local y_offset = -15
	if frametype == "Slider" then
		template = "OptionsSliderTemplate"
		y_offset = -45
	elseif frametype == "Button" then
		template = "GameMenuButtonTemplate"
	end
	
	o[name] = CreateFrame(frametype, "GrowlerOptions_" .. name, o, template)
	o[name].name = name
	o[name].frametype = frametype
	o[name].dbKey = dbKey
	o[name].text = text
	
	if frametype == "CheckButton" then
		o[name]:SetChecked(GrowlerDB.options[dbKey])
		getglobal(o[name]:GetName() .. 'Text'):SetText(text)
	elseif frametype == "Slider" then
		o[name]:SetMinMaxValues(args[1], args[2])
		--o[name]:SetValueStep(args[2]/10)
		getglobal(o[name]:GetName() .. 'Low'):SetText(tostring(args[1]))
		getglobal(o[name]:GetName() .. 'High'):SetText(tostring(args[2]))
		o[name]:SetValue(GrowlerDB.options[dbKey])
		o[name]:SetScript("OnValueChanged",function(self, value)
			getglobal(o[name]:GetName() .. 'Text'):SetText(text .. "\n" .. tostring(math.floor(value*10)/10))
		end)
		o[name]:SetScript("OnMouseUp",function(self,button)
			if button == "RightButton" then
				o[name]:SetValue(Growler.DefaultDB.options[dbKey])
			end
		end)
		getglobal(o[name]:GetName() .. 'Text'):SetText(text .. "\n" .. tostring(math.floor(GrowlerDB.options[dbKey]*10)/10))
	elseif frametype == "Button" then
		o[name].GettingInput = false
		o[name]:SetPropagateKeyboardInput(false)
		o[name].KeysPressed = {}
		o[name].OnKeyDown = args[1]
		o[name].OnKeyUp = args[2]
		getglobal(o[name]:GetName() .. 'Text'):SetText(text)
	end
	
	for k,opt in pairs(o.UsedFields) do
		if opt.frametype == "CheckButton" then
			y_offset = y_offset - 25
		elseif opt.frametype == "Slider" then
			y_offset = y_offset - 55
		else
			y_offset = y_offset - 25
		end
	end
	
	o[name]:ClearAllPoints()
	o[name]:SetPoint("TOPLEFT", 10 + (insetlevel*20), y_offset)
	if handler then o[name]:SetScript("OnClick",handler) end
	o[name]:Show()
	o.UsedFields[name] = o[name]		
end

O.CreateChildPanel = function(name)
	local longname = "GrowlerOptions_" .. name
	O.Category[name] = CreateFrame("FRAME", longname, Growler_Options)
	local o = O.Category[name]
	o.name = name
	o.parent = O.name
	o.UsedFields = {}
	o.okay = O.okay
	o.cancel = O.cancel
	o.refresh = O.cancel
	InterfaceOptions_AddCategory(o)
	return o
end

O.CreateOptionsPanel = function()
	O.Category = {}
	O.UsedFields = {}
	O.name = "Growler"
	InterfaceOptions_AddCategory(O)
	O.okay = function()
		for k,opt in pairs(O.UsedFields) do
			if opt.frametype == "CheckButton" then
				GrowlerDB.options[opt.dbKey] = opt:GetChecked()
			elseif opt.frametype == "Slider" then
				GrowlerDB.options[opt.dbKey] = opt:GetValue()					
			end
		end
		for n,cat in pairs(O.Category) do
			for k,opt in pairs(cat.UsedFields) do
				if opt.frametype == "CheckButton" then
					GrowlerDB.options[opt.dbKey] = opt:GetChecked()
				elseif opt.frametype == "Slider" then
					GrowlerDB.options[opt.dbKey] = opt:GetValue()					
				end
			end
		end
		
		Growler.btn.growl.updatemacro()
		Growler.btn.call.updatemacro()
		Growler.bar.SavePos()
		Growler.bar.ResetPos()
		Growler.SetBindings()
		SL.SetSlash2()
	end		
	O.cancel = function()
		for k,opt in pairs(O.UsedFields) do
			if opt.frametype == "CheckButton" then
				opt:SetChecked(GrowlerDB.options[opt.dbKey])
			elseif opt.frametype == "Slider" then
				opt:SetValue(GrowlerDB.options[opt.dbKey])					
			end
		end
		for n,cat in pairs(O.Category) do
			for k,opt in pairs(cat.UsedFields) do
				if opt.frametype == "CheckButton" then
					opt:SetChecked(GrowlerDB.options[opt.dbKey])
				elseif opt.frametype == "Slider" then
					opt:SetValue(GrowlerDB.options[opt.dbKey])					
				end
			end			
		end
	end		
	O.refresh = O.cancel
	
	local class = tostring(UnitClass("player"))
	local isHunter = (class == L["Hunter"])
	
	if isHunter then
		O.CreateOption("Enable","Enable buttons","enable","CheckButton",O,0,nil)
		O.CreateOption("HideBar","Hide the Growler bar completely","hidebar","CheckButton",O,0,nil)
		O.CreateOption("HideDefault","Hide the default pet bar","hidedefaultbar","CheckButton",O,0,nil)
		O.CreateOption("ShowPortrait","Show portrait and health bar","showportrait","CheckButton",O,0,nil)
		O.CreateOption("HidePetFrame","Hide the default pet frame","hidepetframe","CheckButton",O,0,nil)
		O.CreateOption("DisableWhileRested","Disable alerts when resting and not in combat","disablewhenresting","CheckButton",O,0,nil)
		O.CreateOption("Sound","Enable sound","sound","CheckButton",O,0,nil)
		O.CreateOption("Reminder","Keep making sounds if no action is taken","reminder","CheckButton",O,1,nil)
		O.CreateOption("NonParty","Alert about growl when not in a group","nonparty","CheckButton",O,0,nil)
	end
	O.CreateOption("News","Display messages about recent updates upon login","displaynews","CheckButton",O,0,nil)
	O.CreateOption("Notify","Notify group when someone uses growl","notifygroup","CheckButton",O,0,nil)
	O.CreateOption("Slash2","Let me /growl at people again.. (requires /reload)","usegrowlemote","CheckButton",O,0,nil)
	if isHunter then
		O.CreateOption("Slashmsg","Show a confirmation message when toggling options through slash commands","slashmsg","CheckButton",O,0,nil)
		O.CreateOption("ScreenIcons","Enable blinking icons at the center of the screen","screenicons","CheckButton",O,0,nil)
		O.CreateOption("BarSize","Size of the bar","barsize","Slider",O,0,nil,0.8,2)
		O.CreateOption("SISize","Size of the screen icons","sisize","Slider",O,0,nil,0.5,5)
		O.CreateOption("SavePos","Save the bar position and size between sessions","savepos","CheckButton",O,0,nil)
		
		local o = O.CreateChildPanel("Pet and respec buttons")
		O.CreateOption("ShowPets","Show pet buttons for all of your pets","showpets","CheckButton",o,0,nil)
		O.CreateOption("PetsBottom","Put the pet buttons at the bottom of the bar instead of on top","petsbottom","CheckButton",o,1,nil)
		O.CreateOption("ShowSpecs","Show pet respec buttons","showspecs","CheckButton",o,0,nil)
		O.CreateOption("SpecsBottom","Put the respec buttons at the bottom of the bar instead of on top","specsbottom","CheckButton",o,1,nil)
		O.CreateOption("Dpsremind","Remind you to respec your pet to Ferocity when you are in a group","dpsremind","CheckButton",o,0,nil)
		local o = O.CreateChildPanel("Buttons")
		O.CreateOption("Enable_attack","Show Pet Attack on the bar","enable_attack","CheckButton",o,0,nil)
		O.CreateOption("Enable_command","Show pet movement command button on the bar","enable_command","CheckButton",o,0,nil)
		O.CreateOption("Enable_hotp","Show Heart of the Phoenix on the bar","enable_hotp","CheckButton",o,0,nil)
		O.CreateOption("Enable_laststand","Show Last Stand on the bar","enable_laststand","CheckButton",o,0,nil)
		O.CreateOption("Enable_special","Show family special abilities on the bar","enable_special","CheckButton",o,0,nil)
		O.CreateOption("Enable_growl","Give alerts about Growl","enable_growl","CheckButton",o,0,nil)
		O.CreateOption("Enable_call","Give alerts about Call Pet","enable_call","CheckButton",o,0,nil)
		O.CreateOption("Enable_revive","Give alerts about Revive Pet","enable_revive","CheckButton",o,0,function(self,button,down)
			-- if not o.Enable_revive:GetChecked() then o.Enable_mend:SetChecked(false) end
		end)
		O.CreateOption("Enable_mend","Give alerts about Mend Pet","enable_mend","CheckButton",o,1,function(self,button,down)
			-- if o.Enable_mend:GetChecked() then o.Enable_revive:SetChecked(true) end
		end)
		O.CreateOption("MendPet_Threshhold","Mend Pet Threshhold","mend_threshhold","Slider",o,1,nil,0.1,1)
		O.CreateOption("Enable_playdead","Give alerts about Play Dead","enable_playdead","CheckButton",o,0,nil)
		O.CreateOption("Enable_wakeup","Give alerts about Wake Up","enable_wakeup","CheckButton",o,0,nil)
		O.CreateOption("Enable_assist","Give alerts about putting your pet on assist","enable_assist","CheckButton",o,0,nil)
		O.CreateOption("Enable_defensive","Give alerts about putting your pet on defensive","enable_defensive","CheckButton",o,0,nil)
		O.CreateOption("Enable_passive","Give alerts about putting your pet on passive","enable_passive","CheckButton",o,0,nil)
		O.CreateOption("Snooze","Snooze time (seconds)","snooze","Slider",o,0,nil,5,300)
		
		local o = O.CreateChildPanel("Autocast")
		O.CreateOption("Enable_Autocast","Let Growler control your pets' autocast abilities","enable_autocast","CheckButton",o,0,nil)
		for k,v in pairs(l.pets.autocastAbilities) do
			O.CreateOption("Enable_"..k,"Give alerts about "..k.." and make sure it's on autocast (disable to keep it off)","enable_"..k,"CheckButton",o,0,nil)
		end
		
		local o = O.CreateChildPanel("Automatic actions")
		for k,v in pairs(C.Options["automatic"]) do
			O.CreateOption(v.slash,v.description,k,"CheckButton",o,0,nil)
		end
	end
	
	-- for o,opt in pairs(C.Options.main) do
		-- if isHunter or not opt.hunteronly then
			-- local opttype
			-- if opt.type == "bool" then
				-- opttype = "CheckButton"
			-- elseif opt.type == "number" then
				-- opttype = "Slider"
			-- end
			-- if opttype then
				-- O.CreateOption(opt.slash,opt.description,o,opttype,O,0,nil,opt.min,opt.max)
			-- end
		-- end
	-- end
	-- for c,cat in pairs(C.Options) do
		-- if  c~="main" then
			-- local Child = O.CreateChildPanel(c)
			-- for o,opt in pairs(C.Options[c]) do
				-- if isHunter or not opt.hunteronly then
					-- local opttype
					-- if opt.type == "bool" then
						-- opttype = "CheckButton"
					-- elseif opt.type == "number" then
						-- opttype = "Slider"
					-- end
					-- if opttype then
						-- O.CreateOption(opt.slash,opt.description,o,opttype,Child,0,nil,opt.min,opt.max)
					-- end
				-- end
			-- end
		-- end
	-- end
	
	
	-- O.CreateOption("BindingTest","Binding Test","test","Button",0,function(self,button,down)
		-- self.GettingInput = true
		-- self:SetScript("OnKeyDown",self.OnKeyDown)
		-- self:SetScript("OnKeyUp",self.OnKeyUp)
		-- getglobal(O[self.name]:GetName() .. 'Text'):SetText("Press a key")
	-- end,function(self,key)
		-- if self.GettingInput then 
			-- if key ~= "LCTRL" and
			-- key ~= "LSHIFT" and
			-- key ~= "LALT" and
			-- key ~= "RCTRL" and
			-- key ~= "RSHIFT" and
			-- key ~= "RALT"  then
				-- self:SetScript("OnKeyDown",nil)
				-- table.insert(self.KeysPressed,key)
				-- local output = ""
				-- for i,k in ipairs(self.KeysPressed) do
					-- if i > 1 then output = output .. " + " end
					-- output = output .. k
				-- end
				
				-- getglobal(self:GetName() .. 'Text'):SetText(tostring(output))					
			-- else
				-- local alreadypressed = false
				-- for i,k in ipairs(self.KeysPressed) do
					-- if k == key then alreadypressed = true end
				-- end
				-- if not alreadypressed then table.insert(self.KeysPressed,key) end
			-- end
		-- end
	-- end,function(self,key)
		-- if self.GettingInput then
			-- if key == "LCTRL" or
			-- key == "LSHIFT" or
			-- key == "LALT" or
			-- key == "RCTRL" or
			-- key == "RSHIFT" or
			-- key == "RALT"  then
				-- for i,k in ipairs(self.KeysPressed) do
					-- if k == key then
						-- table.remove(self.KeysPressed,i)
						-- break
					-- end
				-- end
			-- else
				-- self.GettingInput = false
				-- self:SetScript("OnKeyUp",nil)
				-- self.KeysPressed = {}
			-- end
		-- end
	-- end)
		
	O.refresh()
end